/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.jz.dmp.cmdexectool.scheduler.common.task.sql;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.ui.freemarker.FreeMarkerConfigurationFactoryBean;
import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;
import com.alibaba.fastjson.JSONObject;
import com.jz.dmp.cmdexectool.common.constant.CommConstant;
import com.jz.dmp.cmdexectool.common.utils.EncryptionUtils;
import com.jz.dmp.cmdexectool.common.utils.FreeMarkerUtils;
import com.jz.dmp.cmdexectool.controller.bean.DmpProjectConfigInfoDto;
import com.jz.dmp.cmdexectool.entity.DmpSyncingDatasource;
import com.jz.dmp.cmdexectool.mapper.DmpSyncingDatasourceDao;
import com.jz.dmp.cmdexectool.scheduler.common.enums.MyDbType;
import com.jz.dmp.cmdexectool.scheduler.common.process.ResourceInfo;
import com.jz.dmp.cmdexectool.scheduler.common.task.AbstractParameters;
import com.jz.dmp.cmdexectool.scheduler.common.utils.ParameterUtils;
import com.jz.dmp.cmdexectool.scheduler.dao.datasource.MyBaseDataSource;

/**
 * Sql/Hql parameter
 */
public class SqlParameters extends AbstractParameters {
	
	private static Logger logger = LoggerFactory.getLogger(SqlParameters.class);
	
	/**
	 * shell script
	 */
	private String script;

	private String taskAppId;
	
	/**
	* 数据源相关配置
	*/
	private String source;

	/**
	* 目标相关配置
	*/
	private String sink;
	
	/**
	* 环境相关配置
	*/
	private String env;
	
	/**
	* ETL相关配置
	*/
	private String transform;
	
	/**
	 * waterdropScript脚本
	 */
	private String waterdropScript;
	
	/**
	 * 前导语句
	 */
	private List<String> preStatements;
	
	/**
	 * 后导语句
	 */
	private List<String> posStatements;
	
	/**
	 * 执行引擎
	 */
	private String executioEngine;
	
	/**
	 * sql执行语句
	 */
	private String sqlScript;
	
	/**
	 * 源数据源
	 */
	private MyBaseDataSource sourceBaseDataSource;
	
	/**
	 * 目标数据源
	 */
	private MyBaseDataSource targetBaseDataSource;
	
	/**
	 * resource list
	 */
	private List<ResourceInfo> resourceList;

	public SqlParameters(String script, DmpProjectConfigInfoDto projectConfigInfoDto, DmpSyncingDatasourceDao dmpSyncingDatasourceDao, FreeMarkerConfigurationFactoryBean freeMarkerConfigurationFactoryBean, String publicKey) {
		source = "";
		env = "";
		sink = "";
		transform = "";
		
		this.script = script;
		
		JSONObject scriptObj = JSONObject.parseObject(script);
		String outputType = scriptObj.getString("outputType");
		
		String sqlScript = scriptObj.getString("sqlScript");
		//设置sql执行语句
		this.sqlScript = sqlScript;
		//设置执行引擎
		String executioEngine = scriptObj.getString("executioEngine");
		this.executioEngine = executioEngine;
		
		//产生evn模板
		generatorEnvStr(freeMarkerConfigurationFactoryBean);
		//source
		generatorSourceStr(dmpSyncingDatasourceDao, freeMarkerConfigurationFactoryBean, publicKey, scriptObj);
		
		if (CommConstant.OUTPUT_TYPE_CONSOLE.equals(outputType)) {
			//产生console transform and sink
			geneConsoleTransfAndSink(freeMarkerConfigurationFactoryBean);
		}else if (CommConstant.OUTPUT_TYPE_HDFS.equals(outputType)) {
			//产生hdfs transform and sink
			geneHdfsTransfAndSink(freeMarkerConfigurationFactoryBean, scriptObj);
		}else if (CommConstant.OUTPUT_TYPE_TABLE.equals(outputType)) {
			//产生table transform and sink
			geneTableTransfAndSink(dmpSyncingDatasourceDao, freeMarkerConfigurationFactoryBean, scriptObj, publicKey);
		}else if (CommConstant.OUTPUT_TYPE_TOPIC.equals(outputType)) {
			//产生topic transform and sink
			geneTopicTransfAndSink(freeMarkerConfigurationFactoryBean, scriptObj);
		}else if (CommConstant.OUTPUT_TYPE_API.equals(outputType)) {
			//产生api transform and sink
			geneApiTransfAndSink(freeMarkerConfigurationFactoryBean, scriptObj);
		}
		//waterdrop script
		geneWaterdropStr(freeMarkerConfigurationFactoryBean);
		
	}

	/**  
	* @Title: geneWaterdropStr  
	* @Description: TODO(waterdrop script)  
	* @param @param freeMarkerConfig    参数  
	* @return void    返回类型  
	* @throws  
	*/ 
	private void geneWaterdropStr(FreeMarkerConfigurationFactoryBean freeMarkerConfigurationFactoryBean) {
		Map<String, String> waterdropModel = new HashMap<String, String>();
		waterdropModel.put("env", env);
		waterdropModel.put("source", source);
		waterdropModel.put("transform", transform);
		waterdropModel.put("sink", sink);
		this.waterdropScript = FreeMarkerUtils.freemakerNoneWebJson(CommConstant.WATERDROP_FTL, waterdropModel, freeMarkerConfigurationFactoryBean);
	}

	/**  
	* @Title: geneApiTransfAndSink  
	* @Description: TODO(产生api transform and sink)  
	* @param @param freeMarkerConfig
	* @param @param scriptObj
	* @param @param sqlScript    参数  
	* @return void    返回类型  
	* @throws  
	*/ 
	private void geneApiTransfAndSink(FreeMarkerConfigurationFactoryBean freeMarkerConfigurationFactoryBean, JSONObject scriptObj) {
		// 执行引擎是jdbc，不用生成waterdrop
		if (this.executioEngine.equals(CommConstant.EXECUTION_ENGINE_JDBC)) {
			return;
		}
		
		//transform
		//Map<String, String> transformSqlModel = new HashMap<String, String>();
		//transformSqlModel.put("source_table_name", "table_view");
		//transformSqlModel.put("sql", this.sqlScript);
		//transform = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_TRANSFORM_SQL, transformSqlModel, freeMarkerConfig);
		
		JSONObject apiObj = scriptObj.getJSONObject("api");
		String columnFieldsObj = apiObj.getString("columnFields");
		String sqlStr = ParameterUtils.columnMappingHandler(columnFieldsObj);
		
		Map<String, String> transformMappingSqlModel = new HashMap<String, String>();
		transformMappingSqlModel.put("source_table_name", "t");
		transformMappingSqlModel.put("sql", sqlStr);
		transform = transform + FreeMarkerUtils.freemakerNoneWebJson(CommConstant.WATERDROP_FTL_TRANSFORM_SQL, transformMappingSqlModel, freeMarkerConfigurationFactoryBean);
		
		//sink
		Map<String, String> sinkApiModel = new HashMap<String, String>();
		sinkApiModel.put("source_table_name", "t");
		sinkApiModel.put("url", apiObj.getString("apiUrl"));
		sinkApiModel.put("apiKey", apiObj.getString("apiKey"));
		sinkApiModel.put("method", apiObj.getString("method"));
		sinkApiModel.put("signType", apiObj.getString("signType"));
		sinkApiModel.put("authCode", apiObj.getString("authCode"));
		sinkApiModel.put("salt", apiObj.getString("salt"));
		sink = FreeMarkerUtils.freemakerNoneWebJson(CommConstant.WATERDROP_FTL_SINK_API, sinkApiModel, freeMarkerConfigurationFactoryBean);
	}

	/**  
	* @Title: geneTopicTransfAndSink  
	* @Description: TODO(产生topic transform and sink)  
	* @param @param freeMarkerConfig
	* @param @param scriptObj
	* @param @param sqlScript    参数  
	* @return void    返回类型  
	* @throws  
	*/ 
	private void geneTopicTransfAndSink(FreeMarkerConfigurationFactoryBean freeMarkerConfigurationFactoryBean, JSONObject scriptObj) {
		// 执行引擎是jdbc，不用生成waterdrop
		if (this.executioEngine.equals(CommConstant.EXECUTION_ENGINE_JDBC)) {
			return;
		}
		
		//transform
		//Map<String, String> transformSqlModel = new HashMap<String, String>();
		//transformSqlModel.put("source_table_name", "table_view");
		//transformSqlModel.put("sql", this.sqlScript);
		//transform = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_TRANSFORM_SQL, transformSqlModel, freeMarkerConfig);
		//sink
		
		JSONObject topicObj = scriptObj.getJSONObject("topic");
		
		Map<String, String> kafkaModel = new HashMap<String, String>();
		kafkaModel.put("source_table_name", "t");
		kafkaModel.put("topic", topicObj.getString("topic"));
		kafkaModel.put("broker", topicObj.getString("server"));
		sink = FreeMarkerUtils.freemakerNoneWebJson(CommConstant.WATERDROP_FTL_SINK_KAFKA, kafkaModel, freeMarkerConfigurationFactoryBean);
	}

	/**  
	* @Title: geneTableTransfAndSink  
	* @Description: TODO(产生table transform and sink)  
	* @param @param dmpSyncingDatasourceDao
	* @param @param freeMarkerConfig
	* @param @param scriptObj
	* @param @param sqlScript    参数  
	* @return void    返回类型  
	* @throws  
	*/ 
	private void geneTableTransfAndSink(DmpSyncingDatasourceDao dmpSyncingDatasourceDao,
			FreeMarkerConfigurationFactoryBean freeMarkerConfigurationFactoryBean, JSONObject scriptObj, String publicKey) {
		
		JSONObject tableObj = scriptObj.getJSONObject("table");
		
		//设置前导、后导语句
		
		String preImportStatement = tableObj.getString("preImportStatement");
		String postImportStatement = tableObj.getString("postImportStatement");
		
		preStatements = new ArrayList<String>();
		if (StringUtils.isNotEmpty(preImportStatement)) {
			preStatements.add(preImportStatement);
		}
		posStatements = new ArrayList<String>();
		if (StringUtils.isNotEmpty(postImportStatement)) {
			posStatements.add(postImportStatement);
		}
		
		//设置目标执行前导后导语句目标数据源
		Integer targetSourceId = tableObj.getInteger("targetSourceId");
		DmpSyncingDatasource targetSource = dmpSyncingDatasourceDao.queryById(targetSourceId);
		
		String jdbcUrl = targetSource.getJdbcUrl();
		String user = targetSource.getUserName();
		String password = EncryptionUtils.decode(targetSource.getPassword(), publicKey);
		MyDbType myDbType = MyDbType.obtainByIdStr(targetSource.getDatasourceType().toString());
		
		targetBaseDataSource = new MyBaseDataSource();
		targetBaseDataSource.setJdbcUrlDirect(jdbcUrl);
		targetBaseDataSource.setUser(user);
		targetBaseDataSource.setPassword(password);
		targetBaseDataSource.setMyDbType(myDbType);
		
		// 执行引擎是jdbc，不用生成waterdrop
		if (this.executioEngine.equals(CommConstant.EXECUTION_ENGINE_JDBC)) {
			return;
		}
		
		//transform
		//Map<String, String> transformSqlModel = new HashMap<String, String>();
		//transformSqlModel.put("source_table_name", "table_view");
		//transformSqlModel.put("sql", this.sqlScript);
		//transform = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_TRANSFORM_SQL, transformSqlModel, freeMarkerConfig);
		
		String tableFieldsObj = tableObj.getString("tableFields");
		String sqlStr = ParameterUtils.columnMappingHandler(tableFieldsObj);
		
		Map<String, String> transformMappingSqlModel = new HashMap<String, String>();
		transformMappingSqlModel.put("source_table_name", "t");
		transformMappingSqlModel.put("sql", sqlStr);
		transform = transform + FreeMarkerUtils.freemakerNoneWebJson(CommConstant.WATERDROP_FTL_TRANSFORM_SQL, transformMappingSqlModel, freeMarkerConfigurationFactoryBean);
		
		//sink
		
		//targetSource
		if (this.targetBaseDataSource.getMyDbType() == MyDbType.MySQL
				|| this.targetBaseDataSource.getMyDbType() == MyDbType.SQLServer
				|| this.targetBaseDataSource.getMyDbType() == MyDbType.PostgreSQL
				|| this.targetBaseDataSource.getMyDbType() == MyDbType.Oracle
				|| this.targetBaseDataSource.getMyDbType() == MyDbType.DB2
				|| this.targetBaseDataSource.getMyDbType() == MyDbType.INFORMIX) {
			
			String targetTableName = tableObj.getString("targetTableName");
			
			Map<String, String> sinkJdbcModel = new HashMap<String, String>();
			sinkJdbcModel.put("source_table_name", "t");
			sinkJdbcModel.put("save_mode", "overwrite");
			sinkJdbcModel.put("truncate", "true");
			sinkJdbcModel.put("url", jdbcUrl);
			sinkJdbcModel.put("driver", targetSource.getDriverClassName());
			sinkJdbcModel.put("user", user);
			sinkJdbcModel.put("password", password);
			sinkJdbcModel.put("dbtable", targetTableName);
			sink = FreeMarkerUtils.freemakerNoneWebJson(CommConstant.WATERDROP_FTL_SINK_JDBC, sinkJdbcModel, freeMarkerConfigurationFactoryBean);
		}
	}

	/**  
	* @Title: geneHdfsTransfAndSink  
	* @Description: TODO(产生hdfs transform and sink)  
	* @param @param freeMarkerConfig
	* @param @param scriptObj
	* @param @param sqlScript    参数  
	* @return void    返回类型  
	* @throws  
	*/ 
	private void geneHdfsTransfAndSink(FreeMarkerConfigurationFactoryBean freeMarkerConfigurationFactoryBean, JSONObject scriptObj) {
		// 执行引擎是jdbc，不用生成waterdrop
		if (this.executioEngine.equals(CommConstant.EXECUTION_ENGINE_JDBC)) {
			return ;
		}
		
		//transform
		//Map<String, String> transformSqlModel = new HashMap<String, String>();
		//transformSqlModel.put("source_table_name", "table_view");
		//transformSqlModel.put("sql", this.sqlScript);
		//transform = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_TRANSFORM_SQL, transformSqlModel, freeMarkerConfig);
		//sink
		JSONObject hdfsObj = scriptObj.getJSONObject("hdfs");
		String hdfsDir = hdfsObj.getString("hdfsDir");
		
		hdfsDir = hdfsDir+"/"+System.currentTimeMillis();
		
		Map<String, String> hdfsModel = new HashMap<String, String>();
		hdfsModel.put("source_table_name", "t");
		hdfsModel.put("path", hdfsDir);
		hdfsModel.put("save_mode", "overwrite");
		sink = FreeMarkerUtils.freemakerNoneWebJson(CommConstant.WATERDROP_FTL_SINK_HDFS, hdfsModel, freeMarkerConfigurationFactoryBean);
	}

	/**  
	* @Title: geneConsoleTransfAndSink  
	* @Description: TODO(产生console transform and sink)  
	* @param @param freeMarkerConfig
	* @param @param sqlScript    参数  
	* @return void    返回类型  
	* @throws  
	*/ 
	private void geneConsoleTransfAndSink(FreeMarkerConfigurationFactoryBean freeMarkerConfigurationFactoryBean) {
		//执行引擎是jdbc，不用生成waterdrop
		if (this.executioEngine.equals(CommConstant.EXECUTION_ENGINE_JDBC)) {
			return ;
		}
		//transform
		//Map<String, String> transformSqlModel = new HashMap<String, String>();
		//transformSqlModel.put("source_table_name", "table_view");
		//transformSqlModel.put("sql", this.sqlScript);
		//transform = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_TRANSFORM_SQL, transformSqlModel, freeMarkerConfig);
		//sink
		Map<String, String> stdoutModel = new HashMap<String, String>();
		stdoutModel.put("source_table_name", "t");
		sink = FreeMarkerUtils.freemakerNoneWebJson(CommConstant.WATERDROP_FTL_SINK_CONSOLE, stdoutModel, freeMarkerConfigurationFactoryBean);
	}

	/**  
	* @Title: generatorSourceStr  
	* @Description: TODO(生成source模板)  
	* @param @param dmpSyncingDatasourceDao
	* @param @param freeMarkerConfig
	* @param @param publicKey
	* @param @param scriptObj    参数  
	* @return void    返回类型  
	* @throws  
	*/ 
	private void generatorSourceStr(DmpSyncingDatasourceDao dmpSyncingDatasourceDao,
			FreeMarkerConfigurationFactoryBean freeMarkerConfigurationFactoryBean, String publicKey, JSONObject scriptObj) {
		Integer sourceId = scriptObj.getInteger("sourceId");
		DmpSyncingDatasource dmpSyncingDatasource = dmpSyncingDatasourceDao.queryById(sourceId);
		
		String jdbcUrl = dmpSyncingDatasource.getJdbcUrl();
		String user = dmpSyncingDatasource.getUserName();
		String password = EncryptionUtils.decode(dmpSyncingDatasource.getPassword(), publicKey);
		MyDbType myDbType = MyDbType.obtainByIdStr(dmpSyncingDatasource.getDatasourceType().toString());
		
		sourceBaseDataSource = new MyBaseDataSource();
		
		sourceBaseDataSource.setJdbcUrlDirect(jdbcUrl);
		sourceBaseDataSource.setUser(user);
		sourceBaseDataSource.setPassword(password);
		sourceBaseDataSource.setMyDbType(myDbType);
		
		// 如果执行引擎选择的事jdbc，不用生成waterdrop source
		if (this.executioEngine.equals(CommConstant.EXECUTION_ENGINE_JDBC)) {
			
			return;
		}
		
		//jdbc
		if (this.sourceBaseDataSource.getMyDbType() == MyDbType.MySQL
				|| this.sourceBaseDataSource.getMyDbType() == MyDbType.SQLServer
				|| this.sourceBaseDataSource.getMyDbType() == MyDbType.PostgreSQL
				|| this.sourceBaseDataSource.getMyDbType() == MyDbType.Oracle
				|| this.sourceBaseDataSource.getMyDbType() == MyDbType.DB2
				|| this.sourceBaseDataSource.getMyDbType() == MyDbType.INFORMIX) {
			Map<String, String> jdbcModel = new HashMap<String, String>();
			jdbcModel.put("driver", dmpSyncingDatasource.getDriverClassName());
			jdbcModel.put("url", jdbcUrl);
			jdbcModel.put("table","("+this.sqlScript+") AS t");
			jdbcModel.put("result_table_name", "t");
			jdbcModel.put("user", user);
			jdbcModel.put("password", password);
			this.source = FreeMarkerUtils.freemakerNoneWebJson(CommConstant.WATERDROP_FTL_SOURCE_JDBC, jdbcModel,
					freeMarkerConfigurationFactoryBean);
		}else if (this.sourceBaseDataSource.getMyDbType() == MyDbType.Hive) {
			Map<String, String> hiveModel = new HashMap<String, String>();
			hiveModel.put("catalogImplementation", "hive");
			hiveModel.put("pre_sql", "select * from hive_db.hive_table");
			hiveModel.put("result_table_name", "table_view");
			this.source = FreeMarkerUtils.freemakerNoneWebJson(CommConstant.WATERDROP_FTL_SOURCE_HIVE, hiveModel,
					freeMarkerConfigurationFactoryBean);
		}else {
			logger.info("waterdrow，不支持的数据源类型");
			throw new RuntimeException("waterdrow，不支持的数据源");
		}
		
	}

	/**  
	* @Title: generatorEnvStr  
	* @Description: TODO(产生env模板)  
	* @param @param freeMarkerConfig    参数  
	* @return void    返回类型  
	* @throws  
	*/ 
	private void generatorEnvStr(FreeMarkerConfigurationFactoryBean freeMarkerConfigurationFactoryBean) {
		//如果执行引擎选择的事jdbc，不用生成env
		if (this.executioEngine.equals(CommConstant.EXECUTION_ENGINE_JDBC)) {
			return ;
		}
		
		Map<String, String> envModel = new HashMap<String, String>();
		envModel.put("sparkappname", "Waterdrop");
		this.env = FreeMarkerUtils.freemakerNoneWebJson(CommConstant.WATERDROP_FTL_ENV, envModel, freeMarkerConfigurationFactoryBean);
	}

	public String getScript() {
		return script;
	}

	public void setScript(String script) {
		this.script = script;
	}

	public String getTaskAppId() {
		return taskAppId;
	}

	public void setTaskAppId(String taskAppId) {
		this.taskAppId = taskAppId;
	}
	
	public String getSource() {
		return source;
	}

	public void setSource(String source) {
		this.source = source;
	}

	public String getSink() {
		return sink;
	}

	public void setSink(String sink) {
		this.sink = sink;
	}

	public String getEnv() {
		return env;
	}

	public void setEnv(String env) {
		this.env = env;
	}

	public String getTransform() {
		return transform;
	}

	public void setTransform(String transform) {
		this.transform = transform;
	}

	public String getWaterdropScript() {
		return waterdropScript;
	}

	public void setWaterdropScript(String waterdropScript) {
		this.waterdropScript = waterdropScript;
	}

	public List<String> getPreStatements() {
		return preStatements;
	}

	public void setPreStatements(List<String> preStatements) {
		this.preStatements = preStatements;
	}

	public List<String> getPosStatements() {
		return posStatements;
	}

	public void setPosStatements(List<String> posStatements) {
		this.posStatements = posStatements;
	}

	public String getExecutioEngine() {
		return executioEngine;
	}

	public void setExecutioEngine(String executioEngine) {
		this.executioEngine = executioEngine;
	}

	public String getSqlScript() {
		return sqlScript;
	}

	public void setSqlScript(String sqlScript) {
		this.sqlScript = sqlScript;
	}

	public MyBaseDataSource getSourceBaseDataSource() {
		return sourceBaseDataSource;
	}

	public void setSourceBaseDataSource(MyBaseDataSource sourceBaseDataSource) {
		this.sourceBaseDataSource = sourceBaseDataSource;
	}

	public MyBaseDataSource getTargetBaseDataSource() {
		return targetBaseDataSource;
	}

	public void setTargetBaseDataSource(MyBaseDataSource targetBaseDataSource) {
		this.targetBaseDataSource = targetBaseDataSource;
	}

	public List<ResourceInfo> getResourceList() {
		return resourceList;
	}

	public void setResourceList(List<ResourceInfo> resourceList) {
		this.resourceList = resourceList;
	}

	@Override
	public boolean checkParameters() {
		return waterdropScript != null && !waterdropScript.isEmpty();
	}

	@Override
	public List<ResourceInfo> getResourceFilesList() {
		return resourceList;
	}

}
