/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.jz.dmp.cmdexectool.scheduler.dao.datasource;

import com.jz.dmp.cmdexectool.scheduler.common.Constants;
import com.jz.dmp.cmdexectool.scheduler.common.enums.DbType;
import com.jz.dmp.cmdexectool.scheduler.common.utils.StringUtils;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import org.apache.hadoop.hive.conf.HiveConf.ConfVars;

/**
 * data source of hive
 */
public class HiveDataSource extends BaseDataSource {

  /**
   * gets the JDBC url for the data source connection
   * @return jdbc url
   */
  @Override
  public String driverClassSelector() {
    return Constants.ORG_APACHE_HIVE_JDBC_HIVE_DRIVER;
  }

  /**
   * @return db type
   */
  @Override
  public DbType dbTypeSelector() {
    return DbType.HIVE;
  }

  /**
   * build hive jdbc params,append : ?hive_conf_list
   *
   * hive jdbc url template:
   *
   * jdbc:hive2://<host1>:<port1>,<host2>:<port2>/dbName;initFile=<file>;sess_var_list?hive_conf_list#hive_var_list
   *
   * @param otherParams otherParams
   * @return filter otherParams
   */
  @Override
  protected String filterOther(String otherParams) {
    if (StringUtils.isBlank(otherParams)) {
      return "";
    }

    StringBuilder hiveConfListSb = new StringBuilder();
    hiveConfListSb.append("?");
    StringBuilder sessionVarListSb = new StringBuilder();

    String[] otherArray = otherParams.split(";", -1);

    // get the default hive conf var name
    Set<String> hiveConfSet = Stream.of(ConfVars.values()).map(confVars -> confVars.varname)
        .collect(Collectors.toSet());

    for (String conf : otherArray) {
      if (hiveConfSet.contains(conf.split("=")[0])) {
        hiveConfListSb.append(conf).append(";");
      } else {
        sessionVarListSb.append(conf).append(";");
      }
    }

    // remove the last ";"
    if (sessionVarListSb.length() > 0) {
      sessionVarListSb.deleteCharAt(sessionVarListSb.length() - 1);
    }

    if (hiveConfListSb.length() > 0) {
      hiveConfListSb.deleteCharAt(hiveConfListSb.length() - 1);
    }

    return sessionVarListSb.toString() + hiveConfListSb.toString();
  }
  
}
