/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.jz.dmp.cmdexectool.scheduler.server.worker.task.hdfs;

import java.io.File;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardOpenOption;
import java.nio.file.attribute.FileAttribute;
import java.nio.file.attribute.PosixFilePermission;
import java.nio.file.attribute.PosixFilePermissions;
import java.util.Set;

import org.slf4j.Logger;

import com.jz.dmp.cmdexectool.scheduler.common.Constants;
import com.jz.dmp.cmdexectool.scheduler.common.task.AbstractParameters;
import com.jz.dmp.cmdexectool.scheduler.common.task.hdfs.HdfsParameters;
import com.jz.dmp.cmdexectool.scheduler.common.utils.OSUtils;
import com.jz.dmp.cmdexectool.scheduler.server.entity.TaskExecutionContext;
import com.jz.dmp.cmdexectool.scheduler.server.worker.task.AbstractTask;
import com.jz.dmp.cmdexectool.scheduler.server.worker.task.CommandExecuteResult;
import com.jz.dmp.cmdexectool.scheduler.server.worker.task.ShellCommandExecutor;
import com.jz.dmp.cmdexectool.scheduler.server.worker.task.WaterdropCommandExecutor;

/**
 * sql task
 */
public class HdfsTask extends AbstractTask {

	/**
	 * sql parameters
	 */
	private HdfsParameters hdfsParameters;

	/**
	 * shell command executor
	 */
	private ShellCommandExecutor shellCommandExecutor;

	/**
	 * taskExecutionContext
	 */
	private TaskExecutionContext taskExecutionContext;

	/**
	 * constructor
	 * 
	 * @param taskExecutionContext taskExecutionContext
	 * @param logger               logger
	 */
	public HdfsTask(TaskExecutionContext taskExecutionContext, Logger logger) {
		super(taskExecutionContext, logger);

		this.taskExecutionContext = taskExecutionContext;
		this.shellCommandExecutor = new ShellCommandExecutor(this::logHandle, taskExecutionContext, logger);
	}

	@Override
	public void init() {
		logger.info("sql task params {}", taskExecutionContext.getTaskParams());

		//sqlParameters = JSONUtils.parseObject(taskExecutionContext.getTaskParams(), SqlParameters.class);
		hdfsParameters = (HdfsParameters) taskExecutionContext.getParameters();

		if (!hdfsParameters.checkParameters()) {
			throw new RuntimeException("sql task params is not valid");
		}
	}

	@Override
	public void handle() throws Exception {
		try {
			// construct process
			CommandExecuteResult commandExecuteResult = shellCommandExecutor.run(buildCommand());
			setExitStatusCode(commandExecuteResult.getExitStatusCode());
			setAppIds(commandExecuteResult.getAppIds());
			setProcessId(commandExecuteResult.getProcessId());
		} catch (Exception e) {
			logger.error("sql task error", e);
			setExitStatusCode(Constants.EXIT_CODE_FAILURE);
			throw e;
		}
	}

	@Override
	public void cancelApplication(boolean cancelApplication) throws Exception {
		// cancel process
		shellCommandExecutor.cancelApplication();
	}

	/**
	 * create command
	 * 
	 * @return file name
	 * @throws Exception exception
	 */
	private String buildCommand() throws Exception {
		// generate scripts
		String fileName = String.format("%s/%s_node.%s", taskExecutionContext.getExecutePath(),
				taskExecutionContext.getTaskAppId(), OSUtils.isWindows() ? "bat" : "sh");

		Path path = new File(fileName).toPath();

		if (Files.exists(path)) {
			return fileName;
		}

		logger.info("raw script : {}", hdfsParameters.getScript());
		logger.info("task execute path : {}", taskExecutionContext.getExecutePath());

		Set<PosixFilePermission> perms = PosixFilePermissions.fromString(Constants.RWXR_XR_X);
		FileAttribute<Set<PosixFilePermission>> attr = PosixFilePermissions.asFileAttribute(perms);

		if (OSUtils.isWindows()) {
			// Files.createFile(path);
			File file = path.toFile();
			File parentFile = file.getParentFile();
			if (!parentFile.exists()) {
				parentFile.mkdirs();
			}
			file.createNewFile();
		} else {
			Files.createFile(path, attr);
		}

		Files.write(path, hdfsParameters.getCmdScript().getBytes(), StandardOpenOption.APPEND);

		return fileName;
	}

	@Override
	public AbstractParameters getParameters() {
		return hdfsParameters;
	}

}
